@@ -8,11 +8,17 @@ from django.template import RequestContext
 from django.utils.encoding import iri_to_uri
 from django.utils.translation import ugettext_lazy as _
 from django.views.decorators.cache import never_cache
+
+from rosetta.utils.microsofttranslator import Translator, TranslateApiException
+
 from rosetta.conf import settings as rosetta_settings
 from rosetta.polib import pofile
 from rosetta.poutil import find_pos, pagination_range, timestamp_with_timezone
 from rosetta.signals import entry_changed, post_save
 from rosetta.storage import get_storage
+from rosetta.access import can_translate
+
+import json
 import re
 import rosetta
 import unicodedata
@@ -150,7 +156,7 @@ def home(request):
 
                     post_save.send(sender=None, language_code=rosetta_i18n_lang_code, request=request)
                     # Try auto-reloading via the WSGI daemon mode reload mechanism
-                    if  rosetta_settings.WSGI_AUTO_RELOAD and \
+                    if rosetta_settings.WSGI_AUTO_RELOAD and \
                         'mod_wsgi.process_group' in request.environ and \
                         request.environ.get('mod_wsgi.process_group', None) and \
                         'SCRIPT_FILENAME' in request.environ and \
@@ -259,7 +265,8 @@ def home(request):
             rosetta_i18n_app=rosetta_i18n_app,
             page=page,
             query=query,
-            paginator=paginator
+            paginator=paginator,
+            rosetta_i18n_pofile=rosetta_i18n_pofile
         ), context_instance=RequestContext(request))
     else:
         return list_languages(request, do_session_warn=True)
@@ -331,7 +338,7 @@ def list_languages(request, do_session_warn=False):
         languages.append(
             (language[0],
             _(language[1]),
-            sorted([(get_app_name(l), os.path.realpath(l), pofile(l)) for l in  pos], key=lambda app: app[0]),
+            sorted([(get_app_name(l), os.path.realpath(l), pofile(l)) for l in pos], key=lambda app: app[0]),
             )
         )
     try:
@@ -345,7 +352,8 @@ def list_languages(request, do_session_warn=False):
         ADMIN_MEDIA_PREFIX=ADMIN_MEDIA_PREFIX,
         do_session_warn=do_session_warn,
         languages=languages,
-        has_pos=has_pos
+        has_pos=has_pos,
+        rosetta_i18n_catalog_filter=rosetta_i18n_catalog_filter
     ), context_instance=RequestContext(request))
 
 
@@ -370,11 +378,11 @@ def lang_sel(request, langid, idx):
         third_party_apps = rosetta_i18n_catalog_filter in ('all', 'third-party')
         django_apps = rosetta_i18n_catalog_filter in ('all', 'django')
         project_apps = rosetta_i18n_catalog_filter in ('all', 'project')
-        file_ = find_pos(langid, project_apps=project_apps, django_apps=django_apps, third_party_apps=third_party_apps)[int(idx)]
+        file_ = sorted(find_pos(langid, project_apps=project_apps, django_apps=django_apps, third_party_apps=third_party_apps), key=get_app_name)[int(idx)]
 
         storage.set('rosetta_i18n_lang_code', langid)
         storage.set('rosetta_i18n_lang_name', six.text_type([l[1] for l in settings.LANGUAGES if l[0] == langid][0]))
-        storage.set('rosetta_i18n_fn',  file_)
+        storage.set('rosetta_i18n_fn', file_)
         po = pofile(file_)
         for entry in po:
             entry.md5hash = hashlib.new('md5',
@@ -393,18 +401,25 @@ def lang_sel(request, langid, idx):
         return HttpResponseRedirect(reverse('rosetta-home'))
 
 
-def can_translate(user):
-    if not getattr(settings, 'ROSETTA_REQUIRES_AUTH', True):
-        return True
-    if not user.is_authenticated():
-        return False
-    elif user.is_superuser and user.is_staff:
-        return True
-    else:
-        try:
-            from django.contrib.auth.models import Group
-            translators = Group.objects.get(name='translators')
-            return translators in user.groups.all()
-        except Group.DoesNotExist:
-            return False
+@user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)
+def translate_text(request):
+    language_from = request.GET.get('from', None)
+    language_to = request.GET.get('to', None)
+    text = request.GET.get('text', None)
 
+    if language_from == language_to:
+        data = {'success': True, 'translation': text}
+    else:
+        # run the translation:
+        AZURE_CLIENT_ID = getattr(settings, 'AZURE_CLIENT_ID', None)
+        AZURE_CLIENT_SECRET = getattr(settings, 'AZURE_CLIENT_SECRET', None)
+
+        translator = Translator(AZURE_CLIENT_ID, AZURE_CLIENT_SECRET)
+
+        try:
+            translated_text = translator.translate(text, language_to)
+            data = {'success': True, 'translation': translated_text}
+        except TranslateApiException as e:
+            data = {'success': False, 'error': "Translation API Exception: {0}".format(e.message)}
+
+    return HttpResponse(json.dumps(data), mimetype='application/json')
